const fs = require('fs')
const bodyParser = require('body-parser')
const cors = require('cors')
const express = require('express')

const logger = require('./core/logger')


/*******************************************

 SEE http://gitlab.ndda.fr/philc/apixtribe/-/wikis/HOWTOoverview
 To have a quick understanding before doing deeply in source code

 *********************************************/
// check setup*

fs.stat('/etc/nginx/nginx.conf', (err, stats) => {
    if (err === null) {
        logger.debug(`File exists.`)
    } else if (err.code === 'ENOENT') {
        logger.error('Check documentation, nginx have to be installed on this server first, no /etc/nginx/nginx.conf available, install then rerun yarn command.')
        process.exit(1)
    } else {
        logger.error(`Exception: ` + err)
        process.exit(1)
    }
})

// config.js exist in any case from Setup.checkinit();
try {
    const config = require('./tribes/townconf.js')
} catch (err) {
    logger.error('Welcome into apixtribe, you need to init your town by "yarn setup" the first time . Check README\'s project to learn more. more.')
    process.exit(1)
}
// Tribes allow to get local apixtribe instance context
// dataclient .tribeids [] .DOMs [] .routes (plugins {url:name route:path}) .appname {tribeid:[website]}
const dataclient = require('./models/Tribes')
    .init()
logger.info('allowed DOMs to access to this apixtribe server: ', dataclient.DOMs)
const app = express()
app.set('trust proxy', true)
// To set depending of data form or get size to send
app.use(bodyParser.urlencoded(config.bodyparse.urlencoded))
// To set depending of post put json data size to send
app.use(express.json())
app.use(bodyParser.json(config.bodyparse.json))
app.locals.tribeids = dataclient.tribeids
logger.info('app.locals.tribeids', app.locals.tribeids)
// User token authentification and user init user search
const datauser = require('./models/Pagans')
    .init(dataclient.tribeids)
app.locals.tokens = datauser.tokens
logger.info('app.locals.tokens key ', Object.keys(app.locals.tokens))
// Cors management
const corsOptions = {
    origin: (origin, callback) => {
        if (origin === undefined) {
            callback(null, true)
        } else if (origin.indexOf('chrome-extension') > -1) {
            callback(null, true)
        } else {
            // logger.info( 'origin', origin )
            // marchais avant modif eslint const rematch = ( /^https?\:\/\/(.*)\:.*/g ).exec( origin )
            const rematch = (/^https?:\/\/(.*):.*/g)
                .exec(origin)
            // logger.info( rematch )
            let tmp = origin.replace(/http.?:\/\//g, '')
                .split('.')

            if (rematch && rematch.length > 1) tmp = rematch[1].split('.')
            // logger.info( 'tmp', tmp )
            let dom = tmp[tmp.length - 1]
            if (tmp.length > 1) {
                dom = `${tmp[tmp.length - 2]}.${tmp[tmp.length - 1]}`
            }
            logger.info(`origin: ${origin}, dom:${dom}, CORS allowed? : ${dataclient.DOMs.includes(dom)}`)
            if (dataclient.DOMs.includes(dom)) {
                callback(null, true)
            } else {
                logger.info('Origin is not allowed by CORS')
                callback(new Error('Not allowed by CORS'))
            }
        }
    },
    exposedHeaders: Object.keys(config.exposedHeaders)
}
// CORS
app.use(cors(corsOptions))
// Static Routes
app.use(express.static(`${__dirname}/tribes/${config.mayorId}/www/cdn/public`, {
    dotfiles: 'allow'
}))

// Allow to public access a space dev delivered by apixtribe
// this is just a static open route for dev purpose,
// for production, we'll use a nginx static set to /www/app/appname
/* logger.info( `${config.dnsapixtribe}/space/tribeid/website`, dataclient.appname );
Object.keys( dataclient.appname )
	.forEach( cid => {
		dataclient.appname[ cid ].forEach( website => {
			app.use( `/space/${cid}/${website}`, express.static( `${config.tribes}/${cid}/spacedev/${website}` ) );
		} )
	} );
*/
// Routers add any routes from /routes and /plugins
logger.info('Routes available on this apixtribe instance')
logger.info(dataclient.routes)
// prefix only use for dev purpose in production a proxy nginx redirect /app/ to node apixtribe

dataclient.routes.forEach(r => {
    try {
        app.use(r.url, require(r.route))
    } catch (err) {
        logger.error(`WARNING issue with route ${r.route} from ${r.url} check err if route is key then solve err, if not just be aware that this route won't work on your server. If you are not the maintainer and no turn around please contact the email maintainer.`)
        logger.error('raise err-:', err)
    }
})
// Listen web server from config profil (dev prod, other)
app.listen(config.porthttp, () => {
    logger.info(`check in your browser that api works http://${config.dnsapixtribe}:${config.porthttp}`)
})
/* httpServer.setTimeout( config.settimeout );
if( config.withssl === "YES" ) {
	const httpsServer = https.createServer( config.SSLCredentials, app );
	httpsServer.listen( config.port.https, () => {
		logger.info( `check in your browser that api works https://${config.dnsapixtribe}:${config.port.https}` );
	} );
	httpsServer.setTimeout( config.settimeout );
}; */

logger.info("Made with love for people's freedom, enjoy !!!")
